<?php
/**
 * 2024 Sedestral
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@sedestral.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    Sedestral <contact@sedestral.com>
 * @copyright 2024 Sedestral
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * International Registered Trademark & Property of Sedestral
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class SedestralBaseModuleFrontController extends ModuleFrontController
{
    protected $domainTrans;

    public function __construct()
    {
        parent::__construct();
    }

    // Définir le nombre d'articles par page
    const ARTICLES_PER_PAGE = 16;

    /**
     * Vérifie la langue dans l'URL et redirige si nécessaire.
     *
     * @return void
     */
    protected function checkAndRedirectLanguage()
    {
        $lang_iso = $this->context->language->iso_code;

        if (!preg_match('#^/' . $lang_iso . '/#', $_SERVER['REQUEST_URI'])) {
            $languages = Language::getLanguages(true, $this->context->shop->id);

            // Vérifier s'il y a plus d'une langue active
            if (count($languages) > 1) {
                $current_uri = ltrim($_SERVER['REQUEST_URI'], '/');
                $url_with_lang = $this->context->link->getBaseLink() . $lang_iso . '/' . $current_uri;
                Tools::redirect($url_with_lang);
            }
        }
    }

    /**
     * Récupère les paramètres de pagination.
     *
     * @return array [page, total_pages, offset]
     */
    protected function getPagination($whereClause)
    {
        $page = (int) Tools::getValue('page', 1);
        if ($page < 1) {
            $page = 1;
        }

        // Calculer le total des articles
        $total_articles = (int) Db::getInstance()->getValue(
            'SELECT COUNT(*) FROM ' . _DB_PREFIX_ . 'sedestral_post p WHERE ' . $whereClause
        );

        // Calculer le nombre total de pages
        $total_pages = ceil($total_articles / self::ARTICLES_PER_PAGE);
        if ($page > $total_pages && $total_pages > 0) {
            $page = $total_pages;
        }

        // Calculer l'offset
        $offset = self::ARTICLES_PER_PAGE * ($page - 1);

        return [
            'page' => $page,
            'total_pages' => $total_pages,
            'offset' => $offset,
        ];
    }

    /**
     * Récupère les articles selon les critères donnés.
     *
     * @param string $whereClause
     * @param int $offset
     *
     * @return array
     */
    protected function fetchPosts($whereClause, $offset, $limit = self::ARTICLES_PER_PAGE)
    {
        return SedestralPost::fetchPosts($whereClause, $offset, $limit);
    }

    /**
     * Construit le fil d'Ariane.
     *
     * @param array $additionalCrumbs
     *
     * @return void
     */
    protected function buildBreadcrumb($additionalCrumbs = [])
    {
        $enable_breadcrumb = Configuration::get('SEDESTRAL_ENABLE_BREADCRUMB', null, null, null, 1);
        if ($enable_breadcrumb) {
            $breadcrumb = [
                [
                    'count' => 1,
                    'title' => $this->transCompatibility('Accueil'),
                    'url' => $this->context->link->getPageLink('index'),
                ],
                [
                    'count' => 2,
                    'title' => $this->transCompatibility('Blog'),
                    'url' => $this->context->link->getModuleLink('sedestral', 'blog', [], true, $this->context->language->id),
                ],
            ];

            foreach ($additionalCrumbs as $crumb) {
                $breadcrumb[] = $crumb;
            }

            $this->context->smarty->assign('breadcrumb', ['count' => count($breadcrumb), 'links' => $breadcrumb]);
        }
    }

    /**
     * Assigne les métadonnées de la page.
     *
     * @param array $metaData
     *
     * @return void
     */
    protected function assignPageMeta($metaData)
    {
        $page_meta = $this->getTemplateVarPage();

        foreach ($metaData as $key => $value) {
            $page_meta['meta'][$key] = $value;
        }

        $this->context->smarty->assign('page', $page_meta);
    }

    protected function getPublishedCategories()
    {
        $enable_menu = Configuration::get('SEDESTRAL_ENABLE_MENU_CATEGORIES', null, null, null, 1);
        if (!$enable_menu) {
            return [];
        }

        $id_lang = $this->context->language->id;

        // Récupérer les catégories publiées avec leurs titres et routes
        $sql = 'SELECT `title`, `route`, `id_sedestral_category`
            FROM `' . _DB_PREFIX_ . 'sedestral_category`
            WHERE `state` = "publish"
            AND `id_lang` = ' . pSQL($id_lang) . '
            ORDER BY `title` ASC';

        $categories = Db::getInstance()->executeS($sql);

        // Parcourir les catégories pour calculer les URLs
        foreach ($categories as &$category) {
            $category['link'] = $this->context->link->getModuleLink(
                'sedestral',
                'category',
                ['category_rewrite' => $category['route']],
                true,
                $id_lang
            );
        }

        return $categories;
    }

    protected function transCompatibility($key)
    {
        // Depuis PrestaShop 1.7.6, on peut utiliser ->trans()
        // En dessous, on utilise ->l()
        if (version_compare(_PS_VERSION_, '1.7.6.0', '>=')) {
            return $this->trans($key, [], "Modules.sedestral.$this->domainTrans.Front");
        }

        // Sur les anciennes versions, la méthode trans() n’existe pas,
        // on se contente de ->l() (qui ne prend en charge que la clé brute).
        // On ne peut pas gérer le domain ou les params ici.
        return $this->l($key);
    }
}
