<?php
/**
 * 2024 Sedestral
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@sedestral.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    Sedestral <contact@sedestral.com>
 * @copyright 2024 Sedestral
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * International Registered Trademark & Property of Sedestral
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'sedestral/classes/SedestralAbstractCustomAdminController.php';

class AdminSedestralCategoriesController extends SedestralAbstractCustomAdminController
{
    public function __construct()
    {
        $this->table = 'sedestral_category';
        $this->className = 'SedestralCategory';
        $this->lang = false; // Removed multilingual support
        $this->bootstrap = true;
        $this->domainTrans = 'adminsedestralcategoriescontroller';
        parent::__construct();

        // Define the fields to display in the list view
        $this->fields_list = [
            'id_sedestral_category' => [
                'title' => $this->transCompatibility('ID'),
                'align' => 'center',
                'class' => 'fixed-width-xs',
            ],
            'title' => [
                'title' => $this->transCompatibility('Title'),
                'width' => 'auto',
            ],
            'state' => [
                'title' => $this->transCompatibility('State'),
                'type' => 'select',
                'list' => [
                    'draft' => $this->transCompatibility('Draft'),
                    'publish' => $this->transCompatibility('Published'),
                ],
                'filter_type' => 'select',
                'filter_key' => 'a!state',
                'width' => 'auto',
            ],
            'id_lang' => [
                'title' => $this->transCompatibility('Language'),
                'width' => 'auto',
                'callback' => 'displayLanguage',
            ],
            'route' => [
                'title' => $this->transCompatibility('Route'),
                'width' => 'auto',
            ],
            'sitemap_priority' => [
                'title' => $this->transCompatibility('Sitemap Priority'),
                'width' => 'auto',
                'type' => 'float',
                'align' => 'center',
                'callback' => 'displaySitemapPriority',
            ],
        ];

        // Join to get the language name based on id_lang
        $this->_select = 'l.name AS language_name';
        $this->_join = 'LEFT JOIN `' . _DB_PREFIX_ . 'lang` l ON (l.id_lang = a.id_lang)';

        // Set default order
        $this->_defaultOrderBy = 'id_sedestral_category';
        $this->_defaultOrderWay = 'DESC';

        $this->addRowAction('edit');
        $this->addRowAction('delete');
    }

    /**
     * Callback to display the language name.
     *
     * @param string $value
     * @param array $row
     *
     * @return string
     */
    public function displayLanguage($value, $row)
    {
        $language = new Language($row['id_lang']);

        return $language->name;
    }

    /**
     * Callback to display sitemap priority with one decimal.
     *
     * @param float $value
     * @param array $row
     *
     * @return string
     */
    public function displaySitemapPriority($value, $row)
    {
        return number_format((float) $value, 1);
    }

    public function renderForm()
    {
        if (!$this->loadObject(true)) {
            return;
        }

        if (!$this->object->id) {
            $this->object->sitemap_priority = 0.8;
        }

        $states = [
            [
                'id' => 'draft',
                'name' => $this->transCompatibility('Draft'),
            ],
            [
                'id' => 'publish',
                'name' => $this->transCompatibility('Published'),
            ],
        ];

        // Get available languages for the selector
        $languages = Language::getLanguages(false);

        // Get categories for the selector if needed (not required here since this is category controller)
        // $categories = SedestralCategory::getCategories($this->context->language->id);

        $this->fields_form = [
            'legend' => [
                'title' => $this->transCompatibility('Category'),
                'icon' => 'icon-folder',
            ],
            'input' => [
                // Language Selector
                [
                    'type' => 'select',
                    'label' => $this->transCompatibility('Language'),
                    'name' => 'id_lang',
                    'required' => true,
                    'options' => [
                        'query' => $languages,
                        'id' => 'id_lang',
                        'name' => 'name',
                    ],
                    'desc' => $this->transCompatibility('Select the language for this category.'),
                ],
                // Title
                [
                    'type' => 'text',
                    'label' => $this->transCompatibility('Title'),
                    'name' => 'title',
                    'required' => true,
                    'col' => 6,
                    'desc' => $this->transCompatibility('Enter the title of the category. This will be displayed on your blog.'),
                    'size' => 255,
                ],
                // Snippet
                [
                    'type' => 'text',
                    'label' => $this->transCompatibility('Snippet'),
                    'name' => 'snippet',
                    'required' => false,
                    'col' => 6,
                    'desc' => $this->transCompatibility('Provide a short description or snippet for the category. This will be displayed on category listing pages.'),
                    'size' => 500,
                ],
                // Route
                [
                    'type' => 'text',
                    'label' => $this->transCompatibility('Route'),
                    'name' => 'route',
                    'required' => true,
                    'col' => 6,
                    'hint' => 'my-category-name',
                    'desc' => $this->transCompatibility('Enter a URL-friendly route for the category (e.g., "my-category-name"). Use only letters, numbers, hyphens, and underscores. Must be unique within the selected language.'),
                    'size' => 255,
                ],
                // Sitemap Priority
                [
                    'type' => 'text',
                    'label' => $this->transCompatibility('Sitemap Priority'),
                    'name' => 'sitemap_priority',
                    'required' => true,
                    'col' => 6,
                    'hint' => '0.1 - 1.0',
                    'desc' => $this->transCompatibility('Enter the sitemap priority for this category (between 0.1 and 1.0). This helps search engines understand the importance of this page.'),
                    'size' => 4,
                ],
                // State
                [
                    'type' => 'select',
                    'label' => $this->transCompatibility('State'),
                    'name' => 'state',
                    'required' => true,
                    'options' => [
                        'query' => $states,
                        'id' => 'id',
                        'name' => 'name',
                    ],
                    'desc' => $this->transCompatibility('Set the state of the category. "Draft" will save the category without publishing, while "Published" will make it visible on the blog.'),
                ],
            ],
            'submit' => [
                'title' => $this->transCompatibility('Save'),
            ],
        ];

        // Inclure le fichier JavaScript externe
        $this->context->controller->addJS($this->module->getPathUri() . 'views/js/admin.js');

        return parent::renderForm();
    }

    public function postProcess()
    {
        if (Tools::isSubmit('submitAdd' . $this->table)) {
            // Validate sitemap_priority
            $sitemap_priority = Tools::getValue('sitemap_priority');
            if (!is_numeric($sitemap_priority) || $sitemap_priority < 0.1 || $sitemap_priority > 1.0) {
                $this->errors[] = $this->transCompatibility('Sitemap priority must be a number between 0.1 and 1.0.');
            }

            // Validate route uniqueness per language
            $id_lang = (int) Tools::getValue('id_lang');
            $route = Tools::getValue('route');
            $id_sedestral_category = (int) Tools::getValue('id_sedestral_category');

            $sql = 'SELECT COUNT(*) FROM `' . _DB_PREFIX_ . 'sedestral_category` 
                    WHERE `route` = "' . pSQL($route) . '" 
                    AND `id_lang` = ' . $id_lang;

            if ($id_sedestral_category) {
                $sql .= ' AND `id_sedestral_category` != ' . $id_sedestral_category;
            }

            $count = Db::getInstance()->getValue($sql);
            if ($count > 0) {
                $this->errors[] = $this->transCompatibility('The route must be unique within the selected language.');
            }
        }

        parent::postProcess();
    }

    public function processDelete()
    {
        $object = $this->loadObject();

        if (Validate::isLoadedObject($object)) {
            if (!$object->delete()) {
                $this->errors[] = $this->transCompatibility('An error occurred while deleting the category.');
            } else {
                Tools::redirectAdmin(self::$currentIndex . '&conf=1&token=' . $this->token);
            }
        } else {
            $this->errors[] = $this->transCompatibility('Unable to load the category.');
        }
        return true;
    }
}
