<?php
/**
 * 2024 Sedestral
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@sedestral.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    Sedestral <contact@sedestral.com>
 * @copyright 2024 Sedestral
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * International Registered Trademark & Property of Sedestral
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'sedestral/classes/SedestralAbstractCustomAdminController.php';

class AdminSedestralPostsController extends SedestralAbstractCustomAdminController
{
    protected $image_dir;
    protected $image_name;
    public $max_image_size = 2097152; // 2MB

    public function __construct()
    {
        $this->table = 'sedestral_post';
        $this->className = 'SedestralPost';
        $this->lang = false;
        $this->bootstrap = true;
        $this->domainTrans = 'adminsedestralpostscontroller';

        parent::__construct();

        $this->image_dir = _PS_IMG_DIR_ . 'modules/sedestral/';
        $this->image_name = null;

        $this->fields_list = [
            'id_sedestral_post' => ['title' => $this->transCompatibility('ID'), 'width' => 50],
            'title' => ['title' => $this->transCompatibility('Title'), 'width' => 200],
            'snippet' => ['title' => $this->transCompatibility('Snippet'), 'width' => 150],
            'state' => ['title' => $this->transCompatibility('State'), 'width' => 100],
            'id_lang' => ['title' => $this->transCompatibility('Language'), 'width' => 100, 'callback' => 'displayLanguage'],
            'sitemap_priority' => ['title' => $this->transCompatibility('Sitemap Priority'), 'width' => 100],
            'date_published' => [
                'title' => $this->transCompatibility('Publication Date'),
                'width' => 100,
                'type' => 'datetime',
                'align' => 'center',
            ],
        ];

        $this->_select = 'l.name AS language_name';
        $this->_join = 'LEFT JOIN `' . _DB_PREFIX_ . 'lang` l ON (l.id_lang = a.id_lang)';

        $this->addRowAction('edit');
        $this->addRowAction('delete');
        $this->addRowAction('stats');
    }

    public function initToolbar()
    {
        parent::initToolbar();

        // Add the "Statistics" button to the toolbar
        $this->toolbar_btn['stats'] = [
            'href' => self::$currentIndex . '&globalstats&token=' . $this->token,
            'desc' => $this->transCompatibility('Statistics'),
        ];
    }

    public function initProcess()
    {
        parent::initProcess();

        if (Tools::isSubmit('stats' . $this->table)) {
            $this->action = 'stats';
        }

        if (Tools::isSubmit('globalstats')) {
            $this->action = 'globalstats';
        }
    }

    public function processStats()
    {
        $this->renderStats();
    }

    public function processGlobalstats()
    {
        $this->renderGlobalStats();
    }

    public function displayStatsLink($value, $row)
    {
        if (!isset($row)) {
            return '';
        }

        // retrocompatibility entre 1.7 et 8
        if (is_array($row)) {
            if (isset($row['id_sedestral_post'])) {
                $id = $row['id_sedestral_post'];
            } else {
                return '';
            }
        } else {
            $id = $row;
        }

        $token = Tools::getAdminTokenLite('AdminSedestralPosts');
        $href = self::$currentIndex . '&' . $this->identifier . '=' . $id . '&stats' . $this->table . '&token=' . $token;

        $button = '<a style="text-align: left;" href="' . $href . '" class="btn btn-default" title="' . $this->transCompatibility('View Stats') . '">
            <i class="icon-bar-chart"></i> ' . $this->transCompatibility('Statistics') . '
        </a>';

        return $button;
    }

    /**
     * Callback to display the language name.
     */
    public function displayLanguage($value, $row)
    {
        $language = new Language($row['id_lang']);

        return $language->name;
    }

    public function renderForm()
    {
        if (!$this->loadObject(true)) {
            return;
        }

        if (!$this->object->id) {
            $this->object->sitemap_priority = 0.7;
        }

        $states = [
            ['id' => 'draft', 'name' => $this->transCompatibility('Draft')],
            ['id' => 'publish', 'name' => $this->transCompatibility('Publish')],
        ];

        // Get categories for the selector
        $categories = SedestralCategory::getCategories($this->context->language->id);

        $image = $this->object->getCoverThumbnailUrl($this->context, $this->object->cover, 'html');
        $this->object->loadImages($this->context);

        // Build the array of fields in the desired order
        $inputs = [
            // Language Selector
            [
                'type' => 'select',
                'label' => $this->transCompatibility('Language'),
                'name' => 'id_lang',
                'required' => true,
                'options' => [
                    'query' => Language::getLanguages(false),
                    'id' => 'id_lang',
                    'name' => 'name',
                ],
                'desc' => $this->transCompatibility('Select the language for this blog post.'),
            ],

            // Title
            [
                'type' => 'text',
                'label' => $this->transCompatibility('Title'),
                'name' => 'title',
                'required' => true,
                'desc' => $this->transCompatibility('Enter the title of the blog post.'),
                'size' => 500,
            ],

            // Snippet
            [
                'type' => 'text',
                'label' => $this->transCompatibility('Snippet'),
                'name' => 'snippet',
                'required' => true,
                'desc' => $this->transCompatibility('Provide a short summary or snippet for the blog post.'),
                'size' => 1000,
            ],

            // Cover Image
            [
                'type' => 'file',
                'label' => $this->transCompatibility('Cover Image'),
                'name' => 'cover',
                'display_image' => true,
                'image' => $image ?: false,
                'delete_url' => self::$currentIndex . '&' . $this->identifier . '=' . $this->object->id . '&deleteImage=1&token=' . $this->token,
                'desc' => $this->transCompatibility('Upload a cover image for the blog post. Allowed formats: .jpg, .jpeg, .png, .gif. Maximum size: 2MB.'),
            ],

            // Content
            [
                'type' => 'textarea',
                'label' => $this->transCompatibility('Content'),
                'name' => 'contentArticle',
                'default_value' => $this->object->content,
                'required' => true,
                'autoload_rte' => true,
                'desc' => $this->transCompatibility('Write the main content of the blog post. You can use the rich text editor for formatting.'),
            ],

            // Keywords
            [
                'type' => 'text',
                'label' => $this->transCompatibility('Keywords'),
                'name' => 'keywords',
                'desc' => $this->transCompatibility('Enter keywords related to the blog post, separated by commas. This helps in SEO optimization.'),
                'size' => 255,
            ],

            // Route
            [
                'type' => 'text',
                'label' => $this->transCompatibility('Route'),
                'name' => 'route',
                'required' => true,
                'hint' => 'my-post-name',
                'desc' => $this->transCompatibility('Enter a URL-friendly route for the blog post (e.g., "my-post-name"). Use only letters, numbers, hyphens, and underscores. Must be unique within the selected language.'),
                'size' => 255,
            ],

            // Category
            [
                'type' => 'select',
                'label' => $this->transCompatibility('Category'),
                'name' => 'id_sedestral_category',
                'required' => true,
                'options' => [
                    'query' => $categories,
                    'id' => 'id_sedestral_category',
                    'name' => 'title',
                ],
                'desc' => $this->transCompatibility('Select the category under which this blog post will be listed.'),
            ],

            // Author
            [
                'type' => 'text',
                'label' => $this->transCompatibility('Author'),
                'name' => 'author',
                'desc' => $this->transCompatibility('Enter the name of the author of the blog post.'),
                'size' => 255,
            ],

            // State
            [
                'type' => 'select',
                'label' => $this->transCompatibility('State'),
                'name' => 'state',
                'required' => true,
                'options' => [
                    'query' => $states,
                    'id' => 'id',
                    'name' => 'name',
                ],
                'desc' => $this->transCompatibility('Set the state of the blog post. "Draft" will save the post without publishing, while "Publish" will make it live on the blog.'),
            ],

            // Sitemap Priority
            [
                'type' => 'text',
                'label' => $this->transCompatibility('Sitemap Priority'),
                'name' => 'sitemap_priority',
                'value' => '0.7',
                'required' => true,
                'desc' => $this->transCompatibility('Enter the sitemap priority for this blog post (between 0.1 and 1.0). This helps search engines understand the importance of this page.'),
                'hint' => '0.1 - 1.0',
            ],
            [
                'type' => 'datetime', // Using 'text' to allow custom input type
                'label' => $this->transCompatibility('Publication Date and Time'),
                'name' => 'date_published',
                'required' => true, // Set to true as per your requirement
                'desc' => $this->transCompatibility('Select the date and time when the blog post should be published.'),
                'hint' => $this->transCompatibility('Format: YYYY-MM-DDTHH:MM:SS'),
            ],
        ];

        // Define the form
        $this->fields_form = [
            'legend' => [
                'title' => $this->transCompatibility('Post'),
            ],
            'input' => $inputs,
            'submit' => [
                'title' => $this->transCompatibility('Save'),
            ],
        ];

        $this->context->smarty->setTemplateDir([
            _PS_BO_ALL_THEMES_DIR_ . $this->bo_theme . DIRECTORY_SEPARATOR . 'template',
            _PS_OVERRIDE_DIR_ . 'controllers' . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'templates',
        ]);

        // Inclure le fichier JavaScript externe
        $this->context->controller->addJS($this->module->getPathUri() . 'views/js/admin.js');

        return parent::renderForm();
    }

    public function postProcess()
    {
        if (Tools::isSubmit('deleteImage')) {
            if (!$this->loadObject(true)) {
                $this->errors[] = $this->transCompatibility('An error occurred while loading the object.');
            } else {
                $this->deleteCover();
            }
        }

        // Validate unique route per language
        if (Tools::isSubmit('submitAdd' . $this->table)) {
            if (isset($_POST['contentArticle'])) {
                $_POST['content'] = $_POST['contentArticle'];
            }

            // Validate sitemap_priority
            $sitemap_priority = Tools::getValue('sitemap_priority');
            if (!is_numeric($sitemap_priority) || $sitemap_priority < 0.1 || $sitemap_priority > 1.0) {
                $this->errors[] = $this->transCompatibility('Sitemap priority must be a number between 0.1 and 1.0.');
            }

            // Validate unique route per language
            $id_lang = Tools::getValue('id_lang');
            $route = Tools::getValue('route');
            $id_sedestral_post = (int) Tools::getValue('id_sedestral_post');

            $sql = 'SELECT COUNT(*) FROM `' . _DB_PREFIX_ . 'sedestral_post` 
                WHERE `route` = "' . pSQL($route) . '" 
                AND `id_lang` = ' . pSQL($id_lang);

            if ($id_sedestral_post) {
                $sql .= ' AND `id_sedestral_post` != ' . $id_sedestral_post;
            }

            $count = Db::getInstance()->getValue($sql);
            if ($count > 0) {
                $this->errors[] = $this->transCompatibility('The route must be unique within the selected language.');
            }

            // Validate that the selected category matches the selected language
            $id_lang = (int) Tools::getValue('id_lang');
            $id_sedestral_category = (int) Tools::getValue('id_sedestral_category');

            if ($id_sedestral_category) {
                // Retrieve the language of the selected category
                $sql = 'SELECT `id_lang` FROM `' . _DB_PREFIX_ . 'sedestral_category` WHERE `id_sedestral_category` = ' . $id_sedestral_category;
                $category_lang = Db::getInstance()->getValue($sql);

                if ($category_lang && $category_lang != $id_lang) {
                    $this->errors[] = $this->transCompatibility('The selected category must belong to the same language as the post.');
                }
            } else {
                $this->errors[] = $this->transCompatibility('Please select a valid category.');
            }

            // Additional validation for date_published
            $date_published = Tools::getValue('date_published');
            if ($date_published && !$this->validateDateTime($date_published)) {
                $this->errors[] = $this->transCompatibility('The publication date and time is not a valid datetime.');
            }
        }

        parent::postProcess();
    }

    public function processAdd()
    {
        // Call the parent method to save the object
        parent::processAdd();

        // If the object is correctly created, handle the image
        if (Validate::isLoadedObject($this->object)) {
            // Update the image only if a new image is uploaded
            if ($this->uploadImageCover('cover')) {
                $this->object->cover = $this->image_name;
                $this->object->update();
            }
        }

        return true;
    }

    public function processUpdate()
    {
        // Load the object before updating
        if (!$this->loadObject(true)) {
            return;
        }

        // Save the current cover before updating
        $currentCover = $this->object->cover;

        // Update the image only if a new image is uploaded
        if ($this->uploadImageCover('cover')) {
            // Update the 'cover' field with the image file name
            $_POST['cover'] = $this->image_name;
        } else {
            // Keep the current cover
            $_POST['cover'] = $currentCover;
        }

        // Call the parent method to update other fields
        parent::processUpdate();
    }

    public function processDelete()
    {
        // Load the object to delete
        if (!$this->loadObject(true)) {
            return false;
        }

        // Delete associated cover image
        $this->deleteCover();

        // Call the parent method to delete the object
        parent::processDelete();

        return true;
    }

    /**
     * Uploads the cover image.
     *
     * @param string $field_name
     *
     * @return bool
     */
    private function uploadImageCover($field_name)
    {
        if (isset($_FILES[$field_name]) && isset($_FILES[$field_name]['tmp_name']) && !empty($_FILES[$field_name]['tmp_name'])) {
            // Validate the image
            $max_size = $this->max_image_size;
            $error = ImageManager::validateUpload($_FILES[$field_name], $max_size);

            if ($error) {
                $this->errors[] = $error;

                return false;
            }

            // Get the file extension
            $ext = pathinfo($_FILES[$field_name]['name'], PATHINFO_EXTENSION);
            $ext = strtolower($ext);

            // Check if the extension is allowed
            $allowed_ext = ['jpg', 'jpeg', 'png', 'gif'];
            if (!in_array($ext, $allowed_ext)) {
                $this->errors[] = $this->transCompatibility('Invalid image format. Allowed formats are: .jpg, .jpeg, .png, .gif');

                return false;
            }

            // Create the directory if necessary
            if (!file_exists($this->image_dir)) {
                if (!mkdir($this->image_dir, 0755, true)) {
                    $this->errors[] = $this->transCompatibility('Failed to create image directory.');

                    return false;
                }
            }

            // Name the file with the object's ID
            $this->image_name = $this::generateUniqueString() . '.' . $ext;

            // Move the uploaded file to the target directory
            if (!move_uploaded_file($_FILES[$field_name]['tmp_name'], $this->image_dir . $this->image_name)) {
                $this->errors[] = $this->transCompatibility('An error occurred while moving the uploaded file.');

                return false;
            }

            return true;
        }

        return false;
    }

    private function generateUniqueString($length = 32)
    {
        return bin2hex(random_bytes($length / 2));
    }

    /**
     * Deletes the cover image associated with the post.
     */
    private function deleteCover()
    {
        if ($this->object->cover && file_exists($this->image_dir . $this->object->cover)) {
            unlink($this->image_dir . $this->object->cover);
        }
        $this->object->cover = null;
        $this->object->update();
    }

    private function validateDateTime($dateTime)
    {
        // Essaie de créer un objet DateTime à partir de la chaîne
        $d = DateTime::createFromFormat('Y-m-d H:i:s', $dateTime);

        // Vérifie que la création a réussi et que le format est bien valide
        return $d && $d->format('Y-m-d H:i:s') === $dateTime;
    }

    public function renderStats()
    {
        // Get the post ID from the URL parameters
        $id_sedestral_post = (int) Tools::getValue('id_sedestral_post');

        // Load the post object
        $post = new SedestralPost($id_sedestral_post);
        if (!Validate::isLoadedObject($post)) {
            $this->errors[] = $this->transCompatibility('Invalid post');

            return parent::renderList();
        }

        // Fetch the stats data
        $statsDataDaily = $this->getStatsData($id_sedestral_post, 'daily');
        $statsDataMonthly = $this->getStatsData($id_sedestral_post, 'monthly');

        // Determine if there is data to display
        $hasDailyData = array_sum($statsDataDaily) > 0;
        $hasMonthlyData = array_sum($statsDataMonthly) > 0;

        // Define the URL to return to the list of articles
        $back_url = self::$currentIndex . '&token=' . $this->token;

        // Assign data to Smarty template
        $this->context->smarty->assign([
            'post' => $post,
            'statsDataDaily' => json_encode($statsDataDaily),
            'statsDataMonthly' => json_encode($statsDataMonthly),
            'controller_url' => self::$currentIndex . '&token=' . $this->token,
            'last_30_days_label' => $this->transCompatibility('Last 30 Days'),
            'last_12_months_label' => $this->transCompatibility('Last 12 Months'),
            'views_label' => $this->transCompatibility('Views'),
            'date_label' => $this->transCompatibility('Date'),
            'month_label' => $this->transCompatibility('Month'),
            'no_data_label' => $this->transCompatibility('No data available for this period.'),
            'hasDailyData' => $hasDailyData,
            'hasMonthlyData' => $hasMonthlyData,
            'back_url' => $back_url,
            'back_label' => $this->transCompatibility('Back to List'),
        ]);

        // Enqueue the necessary JS libraries (e.g., Chart.js)
        $this->context->controller->addJS('https://www.gstatic.com/charts/loader.js');

        // Set the template to use
        $this->setTemplate('stats.tpl');
    }

    public function renderGlobalStats()
    {
        // Fetch the stats data
        $statsDataDaily = $this->getGlobalStatsData('daily');
        $statsDataMonthly = $this->getGlobalStatsData('monthly');

        // Determine if there is data to display
        $hasDailyData = array_sum($statsDataDaily) > 0;
        $hasMonthlyData = array_sum($statsDataMonthly) > 0;

        // Define the URL to return to the list of articles
        $back_url = self::$currentIndex . '&token=' . $this->token;

        // Assign data to Smarty template
        $this->context->smarty->assign([
            'global_stats_title_label' => $this->transCompatibility('Global Statistics'),
            'statsDataDaily' => json_encode($statsDataDaily),
            'statsDataMonthly' => json_encode($statsDataMonthly),
            'controller_url' => self::$currentIndex . '&token=' . $this->token,
            'last_30_days_label' => $this->transCompatibility('Last 30 Days'),
            'last_12_months_label' => $this->transCompatibility('Last 12 Months'),
            'views_label' => $this->transCompatibility('Views'),
            'date_label' => $this->transCompatibility('Date'),
            'month_label' => $this->transCompatibility('Month'),
            'no_data_label' => $this->transCompatibility('No data available for this period.'),
            'hasDailyData' => $hasDailyData,
            'hasMonthlyData' => $hasMonthlyData,
            'back_url' => $back_url,
            'back_label' => $this->transCompatibility('Back to List'),
        ]);

        // Enqueue the necessary JS libraries
        $this->context->controller->addJS('https://www.gstatic.com/charts/loader.js');

        // Set the template to use
        $this->setTemplate('stats.tpl'); // Reusing stats.tpl
    }

    private function getGlobalStatsData($period = 'daily')
    {
        if ($period == 'daily') {
            // Last 30 days
            $startDate = date('Y-m-d', strtotime('-29 days')); // Include today
            $groupBy = 'DATE(timestamp)';
            $dateFormat = '%Y-%m-%d';
        } else {
            // Last 12 months
            $startDate = date('Y-m-01', strtotime('-11 months')); // Include current month
            $groupBy = 'DATE_FORMAT(timestamp, \'%Y-%m\')';
            $dateFormat = '%Y-%m';
        }

        $sql = 'SELECT DATE_FORMAT(timestamp, \'' . $dateFormat . '\') as period, SUM(count) as total_views
            FROM ' . _DB_PREFIX_ . 'sedestral_post_stats
            WHERE timestamp >= "' . pSQL($startDate) . '"
            GROUP BY ' . $groupBy . '
            ORDER BY timestamp ASC';

        $results = Db::getInstance()->executeS($sql);

        // Initialize data array
        $data = [];

        if ($period == 'daily') {
            // Build an array of the last 30 days
            for ($i = 29; $i >= 0; --$i) {
                $date = date('Y-m-d', strtotime("-$i days"));
                $data[$date] = 0;
            }
        } else {
            // Build an array of the last 12 months
            for ($i = 11; $i >= 0; --$i) {
                $date = date('Y-m', strtotime("-$i months"));
                $data[$date] = 0;
            }
        }

        // Fill in the data from the database results
        foreach ($results as $row) {
            $data[$row['period']] = (int) $row['total_views'];
        }

        // Reverse the data array
        $data = array_reverse($data, true);

        return $data;
    }

    private function getStatsData($id_sedestral_post, $period = 'daily')
    {
        if ($period == 'daily') {
            // Last 30 days
            $startDate = date('Y-m-d', strtotime('-29 days')); // Include today
            $groupBy = 'DATE(timestamp)';
            $dateFormat = '%Y-%m-%d';
        } else {
            // Last 12 months
            $startDate = date('Y-m-01', strtotime('-11 months')); // Include current month
            $groupBy = 'DATE_FORMAT(timestamp, \'%Y-%m\')';
            $dateFormat = '%Y-%m';
        }

        $sql = 'SELECT DATE_FORMAT(timestamp, \'' . $dateFormat . '\') as period, SUM(count) as total_views
        FROM ' . _DB_PREFIX_ . 'sedestral_post_stats
        WHERE post_id = ' . (int) $id_sedestral_post . '
        AND timestamp >= "' . pSQL($startDate) . '"
        GROUP BY ' . $groupBy . '
        ORDER BY timestamp ASC'; // Keep as ASC

        $results = Db::getInstance()->executeS($sql);

        // Initialize data array
        $data = [];

        if ($period == 'daily') {
            // Build an array of the last 30 days
            for ($i = 29; $i >= 0; --$i) {
                $date = date('Y-m-d', strtotime("-$i days"));
                $data[$date] = 0;
            }
        } else {
            // Build an array of the last 12 months
            for ($i = 11; $i >= 0; --$i) {
                $date = date('Y-m', strtotime("-$i months"));
                $data[$date] = 0;
            }
        }

        // Fill in the data from the database results
        foreach ($results as $row) {
            $data[$row['period']] = (int) $row['total_views'];
        }

        // Reverse the data array
        $data = array_reverse($data, true);

        return $data;
    }
}
