<?php
/**
 * 2024 Sedestral
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@sedestral.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    Sedestral <contact@sedestral.com>
 * @copyright 2024 Sedestral
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * International Registered Trademark & Property of Sedestral
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'sedestral/classes/SedestralBaseModuleFrontController.php';

class SedestralApiGetDataModuleFrontController extends SedestralBaseModuleFrontController
{
    public function postProcess()
    {
        header('Content-Type: application/json');

        try {
            $slugParam = trim(Tools::getValue('slug'));
            $idParam = trim(Tools::getValue('id'));
            $remoteId = Tools::getValue('remote_id');
            $type = Tools::strtolower(Tools::getValue('type', 'product'));

            if (($slugParam === '' && $idParam === '') || $remoteId === '') {
                $this->abort(400, 'Paramètres manquants : slug ou id requis, remote_id requis.');
            }

            if (!in_array($type, ['product', 'category', 'blog_article'], true)) {
                $this->abort(400, 'Type invalide : product, category ou blog_article attendu.');
            }

            $tokenObject = $this->getTokenByRemoteId($remoteId);
            if (!$tokenObject || empty($tokenObject->token)) {
                $this->abort(401, 'remote_id invalide ou token introuvable.');
            }

            $idLang = (int) Configuration::get('PS_LANG_DEFAULT');
            $object = null;

            if ($idParam !== '') {
                $parts = explode('-', $idParam, 2);

                if ($type === 'blog_article') {
                    $parts[1] = '0';
                }

                if (count($parts) !== 2 || $parts[0] === '' || $parts[1] === '') {
                    $this->abort(400, 'Paramètre object invalide : format attendu id-lang, ex. 123-2.');
                }

                if (!ctype_digit($parts[0]) || !ctype_digit($parts[1])) {
                    $this->abort(400, 'ID objet et ID lang doivent être numériques.');
                }

                $objectId = (int) $parts[0];
                $idLang = (int) $parts[1];
            } else {
                $langIso = Tools::strtolower(trim(Tools::getValue('lang')));
                if ($langIso !== '') {
                    $isoMap = [];
                    foreach (Language::getLanguages(false) as $lang) {
                        if (isset($lang['iso_code'], $lang['id_lang'])) {
                            $isoMap[$lang['iso_code']] = $lang['id_lang'];
                        }
                    }
                }

                $slugClean = preg_replace('/\.html?$/i', '', strtok(ltrim($slugParam, '/'), '#'));

                if ($type !== 'blog_article') {
                    if (!preg_match('/^(\d+)-/', $slugClean, $m)) {
                        $this->abort(400, 'Slug invalide : identifiant numérique manquant.');
                    }
                    $objectId = (int) $m[1];
                } else {
                    $slugClean = preg_replace('/#.*$/', '', $slugClean);
                    $parts = explode('/', $slugClean);

                    if (count($parts) < 3) {
                        $this->abort(400, 'Slug blog invalide : format attendu /{cat}/{article}.');
                    }

                    $articleRoute = array_pop($parts);
                    $catRoute = array_pop($parts);

                    $category = (new SedestralCategory())->getByRoute($catRoute, $idLang);
                    if (!$category || $category->state !== 'publish') {
                        $this->abort(404, 'Catégorie de blog introuvable ou inactive.');
                    }

                    $sql = (new DbQuery())
                        ->select('id_sedestral_post')
                        ->from('sedestral_post')
                        ->where('route = "' . pSQL($articleRoute) . '"')
                        ->where('id_sedestral_category = ' . (int) $category->id_sedestral_category)
                        ->where('id_lang = ' . $idLang)
                        ->where('state = "publish"')
                        ->where('date_published <= NOW()');

                    if (!$objectId = (int) Db::getInstance()->getValue($sql)) {
                        $this->abort(404, 'Article de blog introuvable ou inactif.');
                    }
                }
            }

            if (!$objectId) {
                $this->abort(400, 'Paramètre object ou slug invalide : identifiant manquant.');
            }

            $route = '';

            switch ($type) {
                case 'product':
                    $object = new Product($objectId, $idLang);
                    $defaultCatId = $object->id_category_default;
                    $category = new Category($defaultCatId, $idLang);
                    $route = $this->context->link->getProductLink(
                        $object->id,
                        $object->link_rewrite,
                        $category->link_rewrite,
                        null,
                        $idLang);
                    break;
                case 'category':
                    $object = new Category($objectId, $idLang);
                    $route = $this->context->link->getCategoryLink($object, null, $idLang);
                    break;
                case 'blog_article':
                    $object = new SedestralPost($objectId);
                    $category = new SedestralCategory($object->id_sedestral_category, $idLang);
                    $route = $this->context->link->getModuleLink(
                        'sedestral',
                        'post',
                        [
                            'category_rewrite' => $category->route,
                            'post_rewrite' => $object->route,
                        ]
                    );
                    break;
            }

            if (!Validate::isLoadedObject($object)) {
                $this->abort(404, ucfirst($type) . ' introuvable ou inactif.');
            }

            if ($type === 'blog_article') {
                $metaTitle = $object->title;
                $metaDescription = $object->snippet ?: '';
                $descriptionShort = '';
                $descriptionLong = $object->content;
            } else {
                $getField = function ($prop) use ($object, $idLang) {
                    if (!property_exists($object, $prop)) {
                        return '';
                    }
                    $val = $object->{$prop};

                    return is_array($val) ? ($val[$idLang] ?? '') : ($val ?? '');
                };

                $metaTitle = $getField('meta_title') ?: $getField('name');
                $metaDescription = $getField('meta_description');
                $descriptionShort = $type === 'product' ? $getField('description_short') : $getField('description');
                $descriptionLong = $type === 'product' ? $getField('description') : $getField('additional_description');

                if ($metaDescription === '') {
                    $metaDescription = $descriptionShort ?: $descriptionLong;
                }
                if ($descriptionLong === '') {
                    $descriptionLong = $descriptionShort;
                }
            }

            $responseObj = [
                'id' => $type === 'blog_article' ? $objectId : $objectId . '-' . $idLang,
                'meta_title' => strip_tags($metaTitle),
                'meta_description' => strip_tags($metaDescription),
                'description_short' => $descriptionShort,
                'description' => $descriptionLong,
                'route' => $route,
            ];

            $module = Module::getInstanceByName('sedestral');
            $version = ($module && property_exists($module, 'version')) ? $module->version : '';

            $payload = [
                'status' => true,
                $type => $responseObj,
                'version' => $version,
            ];

            http_response_code(200);
            exit(json_encode($payload, JSON_UNESCAPED_UNICODE));
        } catch (Exception $e) {
            $this->abort(500, $e->getMessage());
        }

        exit;
    }

    private function abort($code, $error)
    {
        http_response_code($code);
        exit(json_encode(['error' => $error], JSON_UNESCAPED_UNICODE));
    }

    private function getTokenByRemoteId($remoteId)
    {
        $sql = new DbQuery();
        $sql->select('*')
            ->from('sedestral_token')
            ->where('remote_id = "' . pSQL($remoteId) . '"');

        if ($row = Db::getInstance()->getRow($sql)) {
            $token = new SedestralToken();
            $token->hydrate($row);

            return $token;
        }

        return false;
    }
}
