<?php
/**
 * 2024 Sedestral
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@sedestral.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    Sedestral <contact@sedestral.com>
 * @copyright 2024 Sedestral
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * International Registered Trademark & Property of Sedestral
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'sedestral/classes/SedestralBaseModuleFrontController.php';

class SedestralApiGetDatasModuleFrontController extends SedestralBaseModuleFrontController
{
    public function postProcess()
    {
        header('Content-Type: application/json');

        try {
            $remoteId = Tools::getValue('remote_id');
            $tokenObject = $this->getTokenByRemoteId($remoteId);
            if (!$tokenObject || empty($tokenObject->token)) {
                $this->abort(401, 'remote_id invalide ou token introuvable.');
            }

            $offset = max(0, (int) Tools::getValue('offset', 0));
            $limit = (int) Tools::getValue('limit', 50);
            $limit = ($limit < 1 || $limit > 200) ? 50 : $limit;
            $type = Tools::strtolower(Tools::getValue('type', 'product'));

            if (!in_array($type, ['product', 'category', 'blog_article'], true)) {
                $this->abort(400, 'Type invalide : product, category ou blog_article attendu.');
            }

            $iso = Tools::strtolower(trim(Tools::getValue('lang')));
            $isoMap = [];
            foreach (Language::getLanguages(false) as $lang) {
                if (isset($lang['iso_code'], $lang['id_lang'])) {
                    $isoMap[$lang['iso_code']] = $lang['id_lang'];
                }
            }
            $idLang = $iso && isset($isoMap[$iso])
                ? (int) $isoMap[$iso]
                : (int) Configuration::get('PS_LANG_DEFAULT');

            $items = [];

            switch ($type) {
                case 'product':
                    $rows = Product::getProducts(
                        $idLang,
                        $offset,
                        $limit,
                        'id_product',
                        'desc',
                        false,
                        true,
                        $this->context
                    );

                    $link = new Link();
                    $imgType = ImageType::getFormattedName('medium');

                    foreach ($rows as $p) {
                        $idImage = !empty($p['id_image'])
                            ? (int) $p['id_image']
                            : (Product::getCover((int) $p['id_product'])['id_image'] ?? 0);

                        $img = $idImage
                            ? $link->getImageLink($p['link_rewrite'], $idImage, $imgType)
                            : '';

                        if ($img && !preg_match('#^https?://#i', $img)) {
                            $img = 'https://' . ltrim($img, '/');
                        }

                        $items[] = [
                            'id' => $p['id_product'] . '-' . $idLang,
                            'title' => strip_tags($p['name']),
                            'description' => strip_tags($p['description_short']),
                            'imageUrl' => $img,
                        ];
                    }
                    break;
                case 'category':
                    $homeId = (int) Configuration::get('PS_HOME_CATEGORY');
                    $rootId = (int) Configuration::get('PS_ROOT_CATEGORY');
                    $sqlWhere = 'AND c.id_category NOT IN (' . $homeId . ',' . $rootId . ')';

                    $sqlLimit = 'LIMIT ' . (int) $offset . ',' . $limit;

                    $rows = Category::getCategories(
                        $idLang,
                        true,
                        false,
                        $sqlWhere,
                        'ORDER BY c.id_category DESC',
                        $sqlLimit
                    );

                    $link = new Link();
                    $imgType = ImageType::getFormattedName('category');

                    foreach ($rows as $row) {
                        $catObj = new Category((int) $row['id_category'], $idLang);

                        $img = '';
                        if ($catObj->id_image) {
                            $img = $link->getCatImageLink(
                                $catObj->link_rewrite,
                                $catObj->id_image,
                                $imgType
                            );

                            if ($img && !preg_match('#^https?://#i', $img)) {
                                $img = 'https://' . ltrim($img, '/');
                            }
                        }

                        $items[] = [
                            'id' => $row['id_category'] . '-' . $idLang,
                            'title' => strip_tags($row['name']),
                            'description' => strip_tags($row['description'] ?? ''),
                            'imageUrl' => $img,
                        ];
                    }
                    break;
                case 'blog_article':
                    $sql = (new DbQuery())
                        ->select('id_sedestral_post, title, snippet, cover')
                        ->from('sedestral_post')
                        ->where('state = "publish"')
                        ->where('date_published <= NOW()')
                        ->where('id_lang = ' . $idLang)
                        ->orderBy('id_sedestral_post DESC')
                        ->limit($limit, $offset);
                    $rows = Db::getInstance()->executeS($sql);

                    foreach ($rows as $r) {
                        $post = new SedestralPost((int) $r['id_sedestral_post']);
                        $img = $post->getCoverThumbnailUrl($this->context, $post->cover, 'url');

                        $items[] = [
                            'id' => (int) $r['id_sedestral_post'],
                            'title' => strip_tags($r['title']) ?: '',
                            'description' => strip_tags($r['snippet']) ?: '',
                            'imageUrl' => $img,
                        ];
                    }
                    break;
            }

            $module = Module::getInstanceByName('sedestral');
            $version = ($module && property_exists($module, 'version')) ? $module->version : '';

            $payload = [
                'status' => true,
                'type' => $type,
                'offset' => $offset,
                'limit' => $limit,
                'items' => $items,
                'version' => $version,
            ];

            http_response_code(200);
            exit(json_encode($payload, JSON_UNESCAPED_UNICODE));
        } catch (Exception $e) {
            $this->abort(500, $e->getMessage());
        }

        exit;
    }

    private function abort($code, $msg)
    {
        http_response_code($code);
        exit(json_encode(['error' => $msg], JSON_UNESCAPED_UNICODE));
    }

    private function getTokenByRemoteId($remoteId)
    {
        $sql = new DbQuery();
        $sql->select('*')
            ->from('sedestral_token')
            ->where('remote_id = "' . pSQL($remoteId) . '"');

        if ($row = Db::getInstance()->getRow($sql)) {
            $token = new SedestralToken();
            $token->hydrate($row);

            return $token;
        }

        return false;
    }
}
