<?php
/**
 * 2024 Sedestral
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@sedestral.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    Sedestral <contact@sedestral.com>
 * @copyright 2024 Sedestral
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * International Registered Trademark & Property of Sedestral
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'sedestral/classes/SedestralBaseModuleFrontController.php';

class SedestralApiPostDataModuleFrontController extends SedestralBaseModuleFrontController
{
    private const ALLOWED_FIELDS = [
        'meta_title' => 'meta_title',
        'meta_description' => 'meta_description',
        'description_short' => 'description_short',
        'description' => 'description',
    ];

    public function postProcess()
    {
        header('Content-Type: application/json');

        try {
            $idParam = trim(Tools::getValue('id'));
            if ($idParam === '') {
                $this->abort(400, 'Paramètre object manquant');
            }

            $parts = explode('-', $idParam, 2);

            if (count($parts) !== 2 || $parts[0] === '' || $parts[1] === '') {
                $this->abort(400, 'Paramètre object invalide : format attendu id-lang, ex. 123-2.');
            }

            if (!ctype_digit($parts[0]) || !ctype_digit($parts[1])) {
                $this->abort(400, 'ID objet et ID lang doivent être numériques.');
            }

            $objectId = (int) $parts[0];
            $idLang = (int) $parts[1];

            $remoteId = trim(Tools::getValue('remote_id'));
            $type = Tools::strtolower(Tools::getValue('type', 'product'));

            if ($remoteId === '') {
                $this->abort(400, 'remote_id manquant');
            }

            if (!in_array($type, ['product', 'category'], true)) {
                $this->abort(400, 'Type invalide : product|category');
            }

            $tokenObject = $this->getTokenByRemoteId($remoteId);
            if (!$tokenObject || empty($tokenObject->token)) {
                $this->abort(401, 'remote_id invalide ou token introuvable');
            }

            $object = ($type === 'product')
                ? new Product($objectId, $idLang)
                : new Category($objectId, $idLang);

            if (!Validate::isLoadedObject($object)) {
                $this->abort(404, ucfirst($type) . ' introuvable');
            }

            $fieldsIn = [];
            $fieldsArray = Tools::getValue('fields');
            if (is_array($fieldsArray)) {
                $fieldsIn = $fieldsArray;
            }

            foreach (self::ALLOWED_FIELDS as $apiField => $_) {
                $v = Tools::getValue($apiField, null);
                if ($v !== null) {
                    $fieldsIn[$apiField] = $v;
                }
            }

            if (empty($fieldsIn)) {
                $this->abort(400, 'Aucun champ à mettre à jour');
            }

            $convertProp = function ($apiField) use ($type) {
                if ($apiField === 'meta_title') {
                    return ['meta_title', 'name'];
                }

                if ($apiField === 'meta_description') {
                    return ['meta_description'];
                }

                if ($type === 'category') {
                    if ($apiField === 'description_short') {
                        return ['description'];
                    } elseif ($apiField === 'description') {
                        return ['additional_description'];
                    } else {
                        return [$apiField];
                    }
                }

                return [$apiField];
            };

            $applied = [];

            foreach ($fieldsIn as $apiField => $val) {
                if (!isset(self::ALLOWED_FIELDS[$apiField])) {
                    continue;
                }
                foreach ($convertProp($apiField) as $prop) {
                    if (!property_exists($object, $prop)) {
                        continue;
                    }

                    $max = $this->getMaxLength($type, $prop);
                    if ($max && $max > 0) {
                        $plainLen = Tools::strlen($val);
                        if ($plainLen > $max) {
                            $val = $this->truncateHtml($val, $max);
                        }
                    }

                    if (is_array($object->$prop)) {
                        $object->{$prop}[$idLang] = $val;
                    } else {
                        $object->$prop = $val;
                    }
                }

                $applied[] = $apiField;
            }

            if (empty($applied)) {
                $this->abort(400, 'Tous les champs fournis sont interdits');
            }

            if (!$object->save()) {
                throw new Exception('Échec lors de la sauvegarde');
            }

            switch ($type) {
                case 'product':
                    $defaultCatId = $object->id_category_default;
                    $category = new Category($defaultCatId, $idLang);
                    $route = $this->context->link->getProductLink(
                        $object->id,
                        $object->link_rewrite,
                        $category->link_rewrite,
                        null,
                        $idLang);
                    break;

                case 'category':
                    $route = $this->context->link->getCategoryLink($object, null, $idLang);
                    break;

                default:
                    $route = '';
                    break;
            }

            $moduleInstance = Module::getInstanceByName('sedestral');
            $version = ($moduleInstance && property_exists($moduleInstance, 'version')) ? $moduleInstance->version : '';

            http_response_code(200);
            exit(json_encode([
                'status' => true,
                'fields' => $applied,
                'route' => $route,
                'version' => $version,
            ], JSON_UNESCAPED_UNICODE));
        } catch (Exception $e) {
            $this->abort(500, $e->getMessage());
        }
    }

    private function abort($code, $error)
    {
        http_response_code($code);
        exit(json_encode(['error' => $error], JSON_UNESCAPED_UNICODE));
    }

    private function getTokenByRemoteId($remoteId)
    {
        $sql = new DbQuery();
        $sql->select('*')
            ->from('sedestral_token')
            ->where('remote_id = "' . pSQL($remoteId) . '"');

        if ($row = Db::getInstance()->getRow($sql)) {
            $token = new SedestralToken();
            $token->hydrate($row);

            return $token;
        }

        return false;
    }

    private function getMaxLength($type, $prop)
    {
        // Limite configurable (description courte)
        if ($prop === 'description_short') {
            return (int) Configuration::get('PS_PRODUCT_SHORT_DESC_LIMIT', null, null, null, 0);
        }

        // Définition du modèle
        $definition = ($type === 'product')
            ? Product::$definition
            : Category::$definition;

        return isset($definition['fields'][$prop]['size'])
            ? $definition['fields'][$prop]['size']
            : null;
    }

    private function truncateHtml($html, $maxLen)
    {
        if ($html === null || $maxLen <= 0) {
            return '';
        }
        if (mb_strlen($html, 'UTF-8') <= $maxLen) {
            return $html;
        }

        $lo = 0;
        $hi = $maxLen;
        $best = '';

        while ($lo <= $hi) {
            $mid = $this->intdiv_compat($lo + $hi, 2);
            $candidate = mb_substr($html, 0, $mid, 'UTF-8');

            $fixed = $this->tidy_fragment($candidate);

            if (mb_strlen($fixed, 'UTF-8') <= $maxLen) {
                $best = $fixed;
                $lo = $mid + 1;
            } else {
                $hi = $mid - 1;
            }
        }
        return preg_replace('~<(\w+)[^>]*></\1>~', '', $best);
    }

    private function intdiv_compat($a, $b)
    {
        return ($a - ($a % $b)) / $b;
    }

    private function dom_fragment($fragment)
    {
        libxml_use_internal_errors(true);
        $doc = new DOMDocument('1.0', 'UTF-8');

        $wrapper = '<!DOCTYPE html><html><body>' . $fragment . '</body></html>';

        $wrapper = mb_convert_encoding($wrapper, 'HTML-ENTITIES', 'UTF-8');

        $doc->loadHTML($wrapper, LIBXML_HTML_NODEFDTD | LIBXML_HTML_NOIMPLIED);

        $body = $doc->getElementsByTagName('body')->item(0);
        $fixed = '';
        foreach ($body->childNodes as $node) {
            $fixed .= $doc->saveHTML($node);
        }
        libxml_clear_errors();
        return $fixed;
    }

    private function tidy_fragment($fragment)
    {
        if (!class_exists('tidy')) {
            return $this->dom_fragment($fragment);
        }

        $fragment = mb_convert_encoding($fragment, 'HTML-ENTITIES', 'UTF-8');

        $tidy = new tidy();
        $config = ['clean' => true, 'output-html' => true, 'show-body-only' => true, 'wrap' => 0, 'char-encoding' => 'utf8'];
        $tidy->parseString($fragment, $config, 'utf8');
        $tidy->cleanRepair();
        return $tidy;
    }
}
