<?php
/**
 * 2024 Sedestral
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@sedestral.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    Sedestral <contact@sedestral.com>
 * @copyright 2024 Sedestral
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * International Registered Trademark & Property of Sedestral
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class SedestralApiPublishModuleFrontController extends ModuleFrontController
{
    const endpoint = 'https://panel.sedestral.com';

    public function __construct()
    {
        parent::__construct();
    }

    public function postProcess()
    {
        header('Content-Type: application/json');

        try {
            // Only accept POST requests
            if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
                http_response_code(400);
                exit(json_encode(['error' => 'Invalid request method.']));
            }

            // Get POST parameters
            $articleId = Tools::getValue('article_id');
            $remoteId = Tools::getValue('remote_id');

            // Check if parameters are provided
            if (empty($articleId) || empty($remoteId)) {
                http_response_code(400);
                exit(json_encode(['error' => 'Missing parameters.']));
            }

            // Retrieve the token using remoteId
            $tokenObject = $this->getTokenByRemoteId($remoteId);
            if (!$tokenObject || empty($tokenObject->token)) {
                http_response_code(400);
                exit(json_encode(['error' => 'Invalid remoteId or token not found.']));
            }

            $token = $tokenObject->token;

            // Build the API endpoint URL
            $apiUrl = self::endpoint . '/community/article/partner/' . urlencode($articleId) . '/' . urlencode($remoteId) . '/' . urlencode($token);

            // Call the external API
            $apiResponse = $this->callApi($apiUrl);

            if ($apiResponse === false) {
                http_response_code(400);
                exit(json_encode(['error' => 'Failed to fetch article data from API.']));
            }

            // Decode the JSON response
            $articleData = json_decode($apiResponse, true);

            if (json_last_error() !== JSON_ERROR_NONE) {
                http_response_code(400);
                exit(json_encode(['error' => 'Invalid JSON response from API.']));
            }

            $lang = isset($articleData['language']) ? $articleData['language'] : null;
            $id_lang = $this->getIdLangFromIsoCode($lang);

            if (!$id_lang) {
                // Use default language if mapping fails
                $id_lang = (int) Configuration::get('PS_LANG_DEFAULT');
            }

            $category = $this->getCategoryByRoute('default', $id_lang);

            if (!$category) {
                // Create the category
                $category = new SedestralCategory();
                $category->id_lang = $id_lang;
                $category->title = 'default';
                $category->snippet = '';
                $category->state = 'publish';
                $category->route = 'default';
                $category->sitemap_priority = 0.8;

                if (!$category->add()) {
                    http_response_code(400);
                    exit(json_encode(['error' => 'Failed to create category.']));
                }
            }

            // Handle post creation or update
            $post = null;
            $localRemoteId = isset($articleData['remoteId']) ? $articleData['remoteId'] : null;

            if ($localRemoteId) {
                // Try to find the existing post by remote_id
                $post = $this->getPostById($localRemoteId, $id_lang);
            }

            if ($post) {
                // Update the existing post
                $post->id_sedestral_category = $category->id;
            } else {
                // Create a new post
                $post = new SedestralPost();
                $post->id_sedestral_category = $category->id;
                $post->id_lang = $id_lang;
                $post->state = 'draft';
                $post->sitemap_priority = 0.7;

                $post->route = isset($articleData['route']) ? $articleData['route'] : '';

                // New code to check for route uniqueness
                $originalRoute = $post->route;
                $counter = 0;
                while ($this->routeExists($post->route, $id_lang, $post->id)) {
                    ++$counter;
                    $post->route = $originalRoute . '-' . $counter;
                }
            }

            // Map articleData to post fields
            $html = isset($articleData['html']) ? $articleData['html'] : '';
            $post->title = isset($articleData['title']) ? $articleData['title'] : '';
            $post->snippet = isset($articleData['snippet']) ? $articleData['snippet'] : '';
            $post->keywords = isset($articleData['keywords']) ? implode(', ', $articleData['keywords']) : '';

            // Updated author assignment
            if (isset($articleData['writersAccounts'][0]['name'])) {
                $post->author = $articleData['writersAccounts'][0]['name'];
            } elseif (isset($articleData['writersRedactors'][0]['name'])) {
                $post->author = $articleData['writersRedactors'][0]['name'];
            } else {
                $post->author = null;
            }

            // Date published
            if (isset($articleData['time'])) {
                $timestamp = ($articleData['time'] / 1000) - (3 * 3600); // Convert milliseconds to seconds and subtract 3 hours
                $post->date_published = date('Y-m-d H:i:s', $timestamp);
            } else {
                $post->date_published = date('Y-m-d H:i:s', time() - (3 * 3600)); // Subtract 3 hours from the current time
            }

            if (isset($articleData['imageFiles'])) {
                foreach ($articleData['imageFiles'] as $imageFile) {
                    $imageUrl = $this->getImageUrl($imageFile);
                    $imageId = $this->getImageId($imageFile);
                    $isCover = $this->isCoverImage($articleData, $imageFile);
                    if ($imageUrl != null && $imageId != null) {
                        $path = $this->downloadImage($imageUrl, $imageId);
                        if ($path != null) {
                            $html = str_replace($imageUrl, _PS_TMP_IMG_ . $path, $html);
                            if ($isCover) {
                                $post->cover = $path;
                            }
                        }
                    }
                }
            } elseif (isset($articleData['imageFile'])) {
                $imageUrl = $this->getImageUrl($articleData['imageFile']);
                $imageId = $this->getImageId($articleData['imageFile']);
                if ($imageUrl && $imageId) {
                    $post->cover = $this->downloadImage($imageUrl, $imageId);
                }
            }

            $post->content = $html;

            // Save the post
            if ($post->id) {
                if (!$post->update()) {
                    http_response_code(400);
                    exit(json_encode(['error' => 'Failed to update post.']));
                }
            } else {
                if (!$post->add()) {
                    http_response_code(400);
                    exit(json_encode(['error' => 'Failed to create post.']));
                }
            }

            $version = '';
            $module = Module::getInstanceByName('sedestral');
            if ($module) {
                $version = $module->version;
            }

            // Return the id of the created or updated post
            http_response_code(200);
            exit(json_encode(['status' => true, 'post_id' => $post->id, 'version' => $version]));
        } catch (Exception $e) {
            // Log the exception as needed
            http_response_code(500);
            exit(json_encode(['error' => $e->getMessage()]));
        }
    }

    private function getTokenByRemoteId($remoteId)
    {
        // Implement the logic to get the SedestralToken object by remoteId
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('sedestral_token');
        $sql->where('remote_id = "' . pSQL($remoteId) . '"');

        $result = Db::getInstance()->getRow($sql);

        if ($result) {
            $token = new SedestralToken();
            $token->hydrate($result);

            return $token;
        } else {
            return false;
        }
    }

    private function callApi($url)
    {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, ['Accept: application/json']);
        curl_setopt($ch, CURLOPT_TIMEOUT, 20);

        $response = curl_exec($ch);

        if (curl_errno($ch)) {
            curl_close($ch);

            return false;
        }

        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if ($httpCode !== 200) {
            return false;
        }

        return $response;
    }

    private function getCategoryByRoute($name, $id_lang)
    {
        // Implement the logic to find a category by its title and language
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('sedestral_category');
        $sql->where('route = "' . pSQL($name) . '" AND id_lang = ' . pSQL($id_lang));

        $result = Db::getInstance()->getRow($sql);

        if ($result) {
            $category = new SedestralCategory();
            $category->hydrate($result);

            return $category;
        } else {
            return false;
        }
    }

    private function getImageUrl($imageFile)
    {
        // Ensure that 'id' and 'extension' are present
        if (!isset($imageFile['id']) || !isset($imageFile['extension'])) {
            return null;
        }

        // Construct the image URL
        $baseUrl = self::endpoint . '/community/files/';
        return $baseUrl . $imageFile['id'] . '.' . $imageFile['extension'];
    }

    private function getImageId($imageFile)
    {
        if (!isset($imageFile['id'])) {
            return null;
        }
        return $imageFile['id'];
    }

    private function isCoverImage($articleData, $imageFile)
    {
        if (!isset($imageFile['id']) || !isset($articleData['mainImageFileId'])) {
            return false;
        }
        if ($articleData['mainImageFileId'] == null) {
            return false;
        }
        return $imageFile['id'] == $articleData['mainImageFileId'];
    }

    private function downloadImage($imageUrl, $imageId)
    {
        if ($imageUrl == null || $imageId == null) {
            return '';
        }

        if (!filter_var($imageUrl, FILTER_VALIDATE_URL)) {
            return '';
        }

        $destinationDir = _PS_IMG_DIR_ . 'modules/sedestral/';
        if (!is_dir($destinationDir)) {
            if (!mkdir($destinationDir, 0755, true)) {
                return '';
            }
        }

        // Extraire et sécuriser l'extension
        $pathInfo = pathinfo(parse_url($imageUrl, PHP_URL_PATH));
        $secureExtension = isset($pathInfo['extension']) ? strtolower($pathInfo['extension']) : 'jpg';

        // Vérification des extensions autorisées
        $allowedExtensions = ['jpg', 'jpeg', 'png', 'gif'];
        if (!in_array($secureExtension, $allowedExtensions)) {
            return '';
        }

        $name = basename($imageId . '.' . $secureExtension);

        $destinationDir = realpath($destinationDir) . '/';
        $filePath = $destinationDir . $name;

        // Use cURL to download the image
        $ch = curl_init($imageUrl);
        $fp = fopen($filePath, 'wb');

        curl_setopt($ch, CURLOPT_FILE, $fp);
        curl_setopt($ch, CURLOPT_TIMEOUT, 20);

        if (!curl_exec($ch)) {
            fclose($fp);
            unlink($filePath); // Supprime le fichier en cas d'échec
            return '';
        }

        curl_close($ch);
        fclose($fp);

        // Validation du type MIME
        $finfo = new finfo(FILEINFO_MIME_TYPE);
        $mime = $finfo->file($filePath);
        $allowedMimeTypes = ['image/jpeg', 'image/png', 'image/gif'];
        if (!in_array($mime, $allowedMimeTypes)) {
            unlink($filePath);
            return '';
        }

        return $name;
    }

    private function getIdLangFromIsoCode($iso_code)
    {
        if (!$iso_code) {
            return null;
        }

        $iso_code = strtolower($iso_code);
        $languages = Language::getLanguages(false);

        foreach ($languages as $language) {
            if (strtolower($language['iso_code']) == $iso_code) {
                return (int) $language['id_lang'];
            }
        }

        return null; // Language not found
    }

    private function routeExists($route, $id_lang, $excludePostId = null)
    {
        $sql = new DbQuery();
        $sql->select('COUNT(*)');
        $sql->from('sedestral_post');
        $sql->where('route = "' . pSQL($route) . '" AND id_lang = ' . pSQL($id_lang));
        if ($excludePostId) {
            $sql->where('id_sedestral_post != ' . (int) $excludePostId);
        }

        $count = (int) Db::getInstance()->getValue($sql);

        return $count > 0;
    }

    private function getPostById($remoteId, $id_lang)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('sedestral_post');
        $sql->where('id_sedestral_post = "' . pSQL($remoteId) . '" AND id_lang = ' . pSQL($id_lang));

        $result = Db::getInstance()->getRow($sql);

        if ($result) {
            $post = new SedestralPost();
            $post->hydrate($result);

            return $post;
        } else {
            return null;
        }
    }
}
