<?php
/**
 * 2024 Sedestral
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@sedestral.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    Sedestral <contact@sedestral.com>
 * @copyright 2024 Sedestral
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * International Registered Trademark & Property of Sedestral
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class SedestralApiRegisterModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        parent::initContent();
        // Since this is an API endpoint, we don't need to display any template
    }

    public function postProcess()
    {
        header('Content-Type: application/json');

        // Only accept POST requests
        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            http_response_code(400);
            exit(json_encode(['error' => 'Invalid request method.']));
        }

        // Get POST parameters
        $local_id = Tools::getValue('local_id');
        $remote_id = Tools::getValue('remote_id');
        $token = Tools::getValue('token');

        // Check if parameters are provided
        if (empty($local_id) || empty($remote_id) || empty($token)) {
            http_response_code(400);
            exit(json_encode(['error' => 'Missing parameters.']));
        }

        $version = '';
        $module = Module::getInstanceByName('sedestral');
        if ($module) {
            $version = $module->version;
        }

        $shop_name = Configuration::get('PS_SHOP_NAME', null, null, 'Shop');

        // Search for the token object
        $tokenObject = $this->getTokenByLocalId($local_id);
        if (!$tokenObject) {
            $tokenObject = $this->getTokenByToken($local_id);
            if (!$tokenObject) {
                http_response_code(400);
                exit(json_encode(['error' => 'Invalid local_id or remote_id already set.']));
            }

            exit(json_encode(['status' => true, 'api_token' => $tokenObject->api_token, 'shop_name' => $shop_name, 'version' => $version]));
        }

        // Check if remote_id is already set
        if (!empty($tokenObject->remote_id)) {
            http_response_code(400);
            exit(json_encode(['error' => 'remote_id already set for this local_id.']));
        }

        // Update the token object with remote_id and token
        $tokenObject->remote_id = $remote_id;
        $tokenObject->token = $token;
        $tokenObject->api_token = '';

        // Generate API key
        $apiKey = $this->generateApiKey();

        Context::getContext()->employee = new Employee(0);

        $webserviceKey = new WebserviceKey();
        $webserviceKey->id = null;
        $webserviceKey->key = $apiKey;
        $webserviceKey->description = 'Generated by Sedestral for upload medias posts';
        $webserviceKey->active = 1;

        if ($webserviceKey->add()) {
            $resources = ['attachments'];
            $methods = ['GET', 'PUT', 'POST', 'PATCH', 'DELETE', 'HEAD'];

            foreach ($resources as $resource) {
                foreach ($methods as $method) {
                    $data = [
                        'id_webservice_account' => (int) $webserviceKey->id,
                        'resource' => pSQL($resource),
                        'method' => pSQL($method),
                    ];
                    Db::getInstance()->insert('webservice_permission', $data);
                }
            }

            // Save the API token in the token object
            $tokenObject->api_token = $apiKey;
        }

        // Save the token object
        if (!$tokenObject->update()) {
            http_response_code(400);
            exit(json_encode(['error' => 'Failed to update token object.']));
        }

        // Return the api_token in the response
        http_response_code(200);
        exit(json_encode(['status' => true, 'api_token' => $tokenObject->api_token, 'shop_name' => $shop_name, 'version' => $version]));
    }

    protected function generateApiKey()
    {
        // Generate a 32-character random string
        return bin2hex(random_bytes(16));
    }

    protected function getTokenByLocalId($local_id)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('sedestral_token');
        $sql->where('local_id = "' . pSQL($local_id) . '"');
        $sql->where('remote_id = ""');

        $result = Db::getInstance()->getRow($sql);

        if ($result) {
            return new SedestralToken($result['id_token']);
        } else {
            return false;
        }
    }

    protected function getTokenByToken($token)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('sedestral_token');
        $sql->where('token = "' . pSQL($token) . '"');

        $result = Db::getInstance()->getRow($sql);

        if ($result) {
            return new SedestralToken($result['id_token']);
        } else {
            return false;
        }
    }
}
