<?php
/**
 * 2024 Sedestral
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@sedestral.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    Sedestral <contact@sedestral.com>
 * @copyright 2024 Sedestral
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * International Registered Trademark & Property of Sedestral
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'sedestral/classes/SedestralBaseModuleFrontController.php';

class SedestralPostModuleFrontController extends SedestralBaseModuleFrontController
{
    /**
     * Initialise le contenu de la page.
     */
    public function initContent()
    {
        $this->domainTrans = 'post';

        parent::initContent();

        // Vérification et redirection de la langue
        $this->checkAndRedirectLanguage();

        $id_lang = $this->context->language->id;

        // Récupérer les paramètres 'category_rewrite' et 'post_rewrite' depuis l'URL
        $category_rewrite = Tools::getValue('category_rewrite');
        $post_rewrite = Tools::getValue('post_rewrite');

        if (!$category_rewrite || !$post_rewrite) {
            Tools::redirect('index.php');
        }

        // Récupérer la catégorie correspondante
        $category = new SedestralCategory();
        $category = $category->getByRoute($category_rewrite, $id_lang);

        if (!$category) {
            // Catégorie non trouvée, rediriger vers la page 404
            header('HTTP/1.0 404 Not Found');
            $this->setTemplate('errors/404.tpl');

            return;
        }

        // Récupérer l'article correspondant au 'post_rewrite' et à la catégorie
        $post = $this->getPostByRouteAndCategory($post_rewrite, $category, $id_lang);

        if (!$post || $post->state !== 'publish') {
            // Article non trouvé ou non publié, rediriger vers la page 404
            header('HTTP/1.0 404 Not Found');
            $this->setTemplate('errors/404.tpl');

            return;
        }

        // Increment the stats
        SedestralPostStats::incrementCount($post->id_sedestral_post);

        // Get previous and next posts
        $prevNextPosts = $this->getPrevNextPosts($post, $id_lang);

        $post->cover_thumbnail_url = $post->getCoverThumbnailUrl($this->context, $post->cover, 'url');
        $post->loadImages($this->context);

        // Construction du fil d'Ariane avec une étape supplémentaire pour la catégorie
        $this->buildBreadcrumb([
            [
                'count' => 3,
                'title' => $category->title,
                'url' => $this->context->link->getModuleLink('sedestral', 'category', ['category_rewrite' => $category->route], true, $this->context->language->id),
            ],
            [
                'count' => 4,
                'title' => $post->title,
                'url' => '',
            ],
        ]);

        // Récupérer la configuration pour l'affichage de l'en-tête
        $enable_post_header = Configuration::get('SEDESTRAL_ENABLE_POST_HEADER', null, null, null, 1);
        $share_article = Configuration::get('SEDESTRAL_ENABLE_SHARE_ARTICLE', null, null, null, 1);

        // Assignation des variables au template
        $this->context->smarty->assign([
            'module_dir' => $this->module->getPathUri(),
            'linkArticle' => $this->context->link->getModuleLink('sedestral', 'post', [
                'category_rewrite' => $category->route,
                'post_rewrite' => $post->route,
            ]),
            'post' => $post,
            'category' => $category,
            'enable_post_header' => $enable_post_header,
            'share_article' => $share_article,
            'prev_post' => $prevNextPosts['prev'],
            'next_post' => $prevNextPosts['next'],
            'shareLikedIn' => $this->transCompatibility('Share on LinkedIn'),
            'shareEmail' => $this->transCompatibility('Share via Email'),
            'shareOnX' => $this->transCompatibility('Share on X'),
            'shareOnFacebook' => $this->transCompatibility('Share on Facebook'),
            'publishedBy' => $this->transCompatibility('Published by'),
            'on' => $this->transCompatibility('on'),
            'publishedOn' => $this->transCompatibility('Published on'),
            'shareThisArticle' => $this->transCompatibility('Share this article:'),
            'similarPosts' => $this->transCompatibility('Similar posts'),
        ]);

        // Assignation des métadonnées
        $this->assignPageMeta([
            'title' => $post->title,
            'description' => $post->snippet,
            'keywords' => $post->keywords,
        ]);

        // Définir le template à utiliser
        $this->setTemplate('module:sedestral/views/templates/front/post.tpl');
    }

    protected function getPostByRouteAndCategory($route, $category, $id_lang)
    {
        $sql = 'SELECT p.*
                FROM ' . _DB_PREFIX_ . 'sedestral_post p
                WHERE p.route = "' . pSQL($route) . '"
                  AND p.id_sedestral_category = ' . pSQL($category->id_sedestral_category) . '
                  AND p.id_lang = ' . pSQL($id_lang) . '
                  AND p.date_published <= NOW()
                  AND p.state = "publish"';

        $result = Db::getInstance()->getRow($sql);
        if ($result) {
            $post = new SedestralPost();
            $post->hydrate($result);
            $post->cover_thumbnail_url = $post->getCoverThumbnailUrl($this->context, $post->cover, 'url');
            $post->loadImages($this->context);
            return $post;
        }

        return null;
    }

    protected function getPrevNextPosts($currentPost, $id_lang)
    {
        $prevPost = null;
        $nextPost = null;

        $enable_similar_posts = Configuration::get('SEDESTRAL_ENABLE_SIMILAR_POSTS', null, null, null, 1);

        if ($enable_similar_posts) {
            // Get previous post (n-1)
            $sqlPrev = 'SELECT p.*
                FROM ' . _DB_PREFIX_ . 'sedestral_post p
                WHERE p.date_published < "' . pSQL($currentPost->date_published) . '"
                  AND p.date_published <= NOW()
                  AND p.state = "publish"
                  AND p.id_lang = ' . pSQL($id_lang) . '
                  AND p.id_sedestral_category = ' . (int) $currentPost->id_sedestral_category . '
                ORDER BY p.date_published DESC';

            $resultPrev = Db::getInstance()->getRow($sqlPrev);
            if ($resultPrev) {
                $prevPost = new SedestralPost();
                $prevPost->hydrate($resultPrev);
                $prevPost->cover_thumbnail_url = $prevPost->getCoverThumbnailUrl($this->context, $prevPost->cover, 'url');
                $prevPost->total_route = $this->context->link->getModuleLink('sedestral', 'post', [
                    'category_rewrite' => $prevPost->getCategoryRoute(),
                    'post_rewrite' => $prevPost->route,
                ]);
            }

            // Get next post (n+1)
            $sqlNext = 'SELECT p.*
                FROM ' . _DB_PREFIX_ . 'sedestral_post p
                WHERE p.date_published > "' . pSQL($currentPost->date_published) . '"
                  AND p.date_published <= NOW()
                  AND p.state = "publish"
                  AND p.id_lang = ' . pSQL($id_lang) . '
                  AND p.id_sedestral_category = ' . pSQL($currentPost->id_sedestral_category) . '
                ORDER BY p.date_published ASC';

            $resultNext = Db::getInstance()->getRow($sqlNext);
            if ($resultNext) {
                $nextPost = new SedestralPost();
                $nextPost->hydrate($resultNext);
                $nextPost->cover_thumbnail_url = $nextPost->getCoverThumbnailUrl($this->context, $nextPost->cover, 'url');
                $nextPost->total_route = $this->context->link->getModuleLink('sedestral', 'post', [
                    'category_rewrite' => $nextPost->getCategoryRoute(),
                    'post_rewrite' => $nextPost->route,
                ]);
            }
        }

        return ['prev' => $prevPost, 'next' => $nextPost];
    }
}
